@@ -6,6 +6,7 @@
 
 from fwunit.ip import IPSet, IP
 from collections import namedtuple
+import itertools
 
 Rule = namedtuple('Rule', ['src', 'dst', 'app', 'name'])
 
@@ -19,16 +20,24 @@ def to_jsonable(rules):
              'dst': ipset_to_jsonable(r.dst),
              'app': r.app,
              'name': r.name}
-            for r in rules]
+            for r in itertools.chain(*rules.itervalues())]
 
 
-def ipset_from_jsonable(ipset):
-    return IPSet([IP(pfx) for pfx in ipset])
+def ipset_from_jsonable(ipset, _cache={}):
+    ipset = tuple(ipset)
+    if ipset not in _cache:
+        _cache[ipset] = rv = IPSet([IP(pfx) for pfx in ipset])
+        return rv
+    return _cache[ipset]
 
 
 def from_jsonable(rules):
-    return [Rule(src=ipset_from_jsonable(r['src']),
-                 dst=ipset_from_jsonable(r['dst']),
-                 app=r['app'],
-                 name=r['name'])
-            for r in rules]
+    by_app = {}
+    for d in rules:
+        r = Rule(src=ipset_from_jsonable(d['src']),
+                 dst=ipset_from_jsonable(d['dst']),
+                 app=d['app'],
+                 name=d['name'])
+        app = r.app
+        by_app.setdefault(app, []).append(r)
+    return by_app
